@@ -34,6 +34,17 @@ type CSharpPackageInfo struct {
 	ProjectReferences      []string          `json:"projectReferences,omitempty"`
 	// Determines whether to make single-return-value methods return an output object or the single value.
 	LiftSingleValueMethodReturns bool `json:"liftSingleValueMethodReturns,omitempty"`
+
+	// The root namespace used for the package. This defaults to `Pulumi`.
+	RootNamespace string `json:"rootNamespace,omitempty"`
+}
+
+// Returns the root namespace, or "Pulumi" if not provided.
+func (info *CSharpPackageInfo) GetRootNamespace() string {
+	if r := info.RootNamespace; r != "" {
+		return r
+	}
+	return "Pulumi"
 }
 
 // Importer implements schema.Language for .NET.
