@@ -24,10 +24,14 @@ import (
 	"github.com/zclconf/go-cty/cty"
 )
 
+func assertConvertibleFrom(t *testing.T, to, from Type) {
+	assert.NotEqual(t, NoConversion, to.ConversionFrom(from))
+}
+
 func TestBindLiteral(t *testing.T) {
 	expr, diags := BindExpressionText("false", nil, hcl.Pos{})
 	assert.Len(t, diags, 0)
-	assert.Equal(t, BoolType, expr.Type())
+	assertConvertibleFrom(t, BoolType, expr.Type())
 	lit, ok := expr.(*LiteralValueExpression)
 	assert.True(t, ok)
 	assert.Equal(t, cty.False, lit.Value)
@@ -35,7 +39,7 @@ func TestBindLiteral(t *testing.T) {
 
 	expr, diags = BindExpressionText("true", nil, hcl.Pos{})
 	assert.Len(t, diags, 0)
-	assert.Equal(t, BoolType, expr.Type())
+	assertConvertibleFrom(t, BoolType, expr.Type())
 	lit, ok = expr.(*LiteralValueExpression)
 	assert.True(t, ok)
 	assert.Equal(t, cty.True, lit.Value)
@@ -43,7 +47,7 @@ func TestBindLiteral(t *testing.T) {
 
 	expr, diags = BindExpressionText("0", nil, hcl.Pos{})
 	assert.Len(t, diags, 0)
-	assert.Equal(t, NumberType, expr.Type())
+	assertConvertibleFrom(t, NumberType, expr.Type())
 	lit, ok = expr.(*LiteralValueExpression)
 	assert.True(t, ok)
 	assert.True(t, cty.NumberIntVal(0).RawEquals(lit.Value))
@@ -51,7 +55,7 @@ func TestBindLiteral(t *testing.T) {
 
 	expr, diags = BindExpressionText("3.14", nil, hcl.Pos{})
 	assert.Len(t, diags, 0)
-	assert.Equal(t, NumberType, expr.Type())
+	assertConvertibleFrom(t, NumberType, expr.Type())
 	lit, ok = expr.(*LiteralValueExpression)
 	assert.True(t, ok)
 	assert.True(t, cty.MustParseNumberVal("3.14").RawEquals(lit.Value))
@@ -59,7 +63,7 @@ func TestBindLiteral(t *testing.T) {
 
 	expr, diags = BindExpressionText(`"foo"`, nil, hcl.Pos{})
 	assert.Len(t, diags, 0)
-	assert.Equal(t, StringType, expr.Type())
+	assertConvertibleFrom(t, StringType, expr.Type())
 	template, ok := expr.(*TemplateExpression)
 	assert.True(t, ok)
 	assert.Len(t, template.Parts, 1)
@@ -131,7 +135,7 @@ func TestBindBinaryOp(t *testing.T) {
 		t.Run(c.x, func(t *testing.T) {
 			expr, diags := BindExpressionText(c.x, scope, hcl.Pos{})
 			assert.Len(t, diags, 0)
-			assert.Equal(t, c.t, expr.Type())
+			assertConvertibleFrom(t, c.t, expr.Type())
 			_, ok := expr.(*BinaryOpExpression)
 			assert.True(t, ok)
 			assert.Equal(t, c.x, fmt.Sprintf("%v", expr))
@@ -163,7 +167,7 @@ func TestBindConditional(t *testing.T) {
 		t.Run(c.x, func(t *testing.T) {
 			expr, diags := BindExpressionText(c.x, scope, hcl.Pos{})
 			assert.Len(t, diags, 0)
-			assert.Equal(t, c.t, expr.Type())
+			assertConvertibleFrom(t, c.t, expr.Type())
 			_, ok := expr.(*ConditionalExpression)
 			assert.True(t, ok)
 			assert.Equal(t, c.x, fmt.Sprintf("%v", expr))
@@ -224,7 +228,7 @@ func TestBindFor(t *testing.T) {
 		t.Run(c.x, func(t *testing.T) {
 			expr, diags := BindExpressionText(c.x, scope, hcl.Pos{})
 			assert.Len(t, diags, 0)
-			assert.Equal(t, c.t, expr.Type())
+			assertConvertibleFrom(t, c.t, expr.Type())
 			_, ok := expr.(*ForExpression)
 			assert.True(t, ok)
 			assert.Equal(t, c.x, fmt.Sprintf("%v", expr))
@@ -280,7 +284,7 @@ func TestBindFunctionCall(t *testing.T) {
 		t.Run(c.x, func(t *testing.T) {
 			expr, diags := BindExpressionText(c.x, scope, hcl.Pos{})
 			assert.Len(t, diags, 0)
-			assert.Equal(t, c.t, expr.Type())
+			assertConvertibleFrom(t, c.t, expr.Type())
 			_, ok := expr.(*FunctionCallExpression)
 			assert.True(t, ok)
 			assert.Equal(t, c.x, fmt.Sprintf("%v", expr))
@@ -347,7 +351,7 @@ func TestBindIndex(t *testing.T) {
 		t.Run(c.x, func(t *testing.T) {
 			expr, diags := BindExpressionText(c.x, scope, hcl.Pos{})
 			assert.Len(t, diags, 0)
-			assert.Equal(t, c.t, expr.Type())
+			assertConvertibleFrom(t, c.t, expr.Type())
 			_, ok := expr.(*IndexExpression)
 			assert.True(t, ok)
 			assert.Equal(t, c.x, fmt.Sprintf("%v", expr))
@@ -388,7 +392,7 @@ func TestBindObjectCons(t *testing.T) {
 		t.Run(c.x, func(t *testing.T) {
 			expr, diags := BindExpressionText(c.x, scope, hcl.Pos{})
 			assert.Len(t, diags, 0)
-			assert.Equal(t, c.t, expr.Type())
+			assertConvertibleFrom(t, c.t, expr.Type())
 			_, ok := expr.(*ObjectConsExpression)
 			assert.True(t, ok)
 			assert.Equal(t, c.x, fmt.Sprintf("%v", expr))
@@ -441,7 +445,7 @@ func TestBindRelativeTraversal(t *testing.T) {
 		t.Run(c.x, func(t *testing.T) {
 			expr, diags := BindExpressionText(c.x, scope, hcl.Pos{})
 			assert.Len(t, diags, 0)
-			assert.Equal(t, c.t, expr.Type())
+			assertConvertibleFrom(t, c.t, expr.Type())
 			_, ok := expr.(*RelativeTraversalExpression)
 			assert.True(t, ok)
 			assert.Equal(t, c.x, fmt.Sprintf("%v", expr))
@@ -511,7 +515,7 @@ func TestBindScopeTraversal(t *testing.T) {
 		t.Run(c.x, func(t *testing.T) {
 			expr, diags := BindExpressionText(c.x, scope, hcl.Pos{})
 			assert.Len(t, diags, 0)
-			assert.Equal(t, c.t, expr.Type())
+			assertConvertibleFrom(t, c.t, expr.Type())
 			_, ok := expr.(*ScopeTraversalExpression)
 			assert.True(t, ok)
 			assert.Equal(t, c.x, fmt.Sprintf("%v", expr))
@@ -578,7 +582,7 @@ func TestBindSplat(t *testing.T) {
 		t.Run(c.x, func(t *testing.T) {
 			expr, diags := BindExpressionText(c.x, scope, hcl.Pos{})
 			assert.Len(t, diags, 0)
-			assert.Equal(t, c.t, expr.Type())
+			assertConvertibleFrom(t, c.t, expr.Type())
 			_, ok := expr.(*SplatExpression)
 			assert.True(t, ok)
 			assert.Equal(t, c.x, fmt.Sprintf("%v", expr))
@@ -639,7 +643,7 @@ func TestBindTemplate(t *testing.T) {
 		t.Run(c.x, func(t *testing.T) {
 			expr, diags := BindExpressionText(c.x, scope, hcl.Pos{})
 			assert.Len(t, diags, 0)
-			assert.Equal(t, c.t, expr.Type())
+			assertConvertibleFrom(t, c.t, expr.Type())
 
 			var ok bool
 			switch c.xt.(type) {
@@ -674,7 +678,7 @@ func TestBindTupleCons(t *testing.T) {
 		t.Run(c.x, func(t *testing.T) {
 			expr, diags := BindExpressionText(c.x, scope, hcl.Pos{})
 			assert.Len(t, diags, 0)
-			assert.Equal(t, c.t, expr.Type())
+			assertConvertibleFrom(t, c.t, expr.Type())
 			_, ok := expr.(*TupleConsExpression)
 			assert.True(t, ok)
 			assert.Equal(t, c.x, fmt.Sprintf("%v", expr))
@@ -708,7 +712,7 @@ func TestBindUnaryOp(t *testing.T) {
 		t.Run(c.x, func(t *testing.T) {
 			expr, diags := BindExpressionText(c.x, scope, hcl.Pos{})
 			assert.Len(t, diags, 0)
-			assert.Equal(t, c.t, expr.Type())
+			assertConvertibleFrom(t, c.t, expr.Type())
 			_, ok := expr.(*UnaryOpExpression)
 			assert.True(t, ok)
 			assert.Equal(t, c.x, fmt.Sprintf("%v", expr))
