@@ -80,6 +80,9 @@ func (b *binder) bindInvokeSignature(args []model.Expression) (model.StaticFunct
 		return b.zeroSignature(), hcl.Diagnostics{unknownFunction(token, tokenRange)}
 	}
 
+	if len(args) < 2 {
+		return b.zeroSignature(), hcl.Diagnostics{errorf(tokenRange, "missing second arg")}
+	}
 	sig, err := b.signatureForArgs(fn, args[1])
 	if err != nil {
 		diag := hcl.Diagnostics{errorf(tokenRange, "Invoke binding error: %v", err)}
@@ -114,7 +117,7 @@ func (b *binder) zeroSignature() model.StaticFunctionSignature {
 }
 
 func (b *binder) signatureForArgs(fn *schema.Function, args model.Expression) (model.StaticFunctionSignature, error) {
-	if b.useOutputVersion(fn, args) {
+	if args != nil && b.useOutputVersion(fn, args) {
 		return b.outputVersionSignature(fn)
 	}
 	return b.regularSignature(fn), nil
