@@ -535,3 +535,51 @@ func TestReplaceOnChanges(t *testing.T) {
 		})
 	}
 }
+
+func TestValidateTypeToken(t *testing.T) {
+	cases := []struct {
+		name          string
+		input         string
+		expectError   bool
+		allowedExtras []string
+	}{
+		{
+			name:  "valid",
+			input: "example::typename",
+		},
+		{
+			name:        "invalid",
+			input:       "xyz::typename",
+			expectError: true,
+		},
+		{
+			name:  "valid-has-subsection",
+			input: "example:index:typename",
+		},
+		{
+			name:        "invalid-has-subsection",
+			input:       "not:index:typename",
+			expectError: true,
+		},
+		{
+			name:          "allowed-extras-valid",
+			input:         "other:index:typename",
+			allowedExtras: []string{"other"},
+		},
+	}
+	for _, c := range cases {
+		t.Run(c.name, func(t *testing.T) {
+			spec := &PackageSpec{Name: "example"}
+			allowed := map[string]bool{"example": true}
+			for _, e := range c.allowedExtras {
+				allowed[e] = true
+			}
+			errors := spec.validateTypeToken(allowed, "type", c.input)
+			if c.expectError {
+				assert.True(t, errors.HasErrors())
+			} else {
+				assert.False(t, errors.HasErrors())
+			}
+		})
+	}
+}
