@@ -161,3 +161,27 @@ func PlainType(t schema.Type) schema.Type {
 func ResolvedType(t schema.Type) schema.Type {
 	return resolvedType(t, true)
 }
+
+// If a helper function needs to be invoked to provide default values for a
+// plain type. The provided map cannot be reused.
+func IsProvideDefaultsFuncRequired(t schema.Type) bool {
+	return isProvideDefaultsFuncRequiredHelper(t, map[string]bool{})
+}
+
+func isProvideDefaultsFuncRequiredHelper(t schema.Type, seen map[string]bool) bool {
+	if seen[t.String()] {
+		return false
+	}
+	seen[t.String()] = true
+	t = UnwrapType(t)
+	object, ok := t.(*schema.ObjectType)
+	if !ok {
+		return false
+	}
+	for _, p := range object.Properties {
+		if p.DefaultValue != nil || isProvideDefaultsFuncRequiredHelper(p.Type, seen) {
+			return true
+		}
+	}
+	return false
+}
