@@ -36,37 +36,48 @@ import (
 	pulumirpc "github.com/pulumi/pulumi/sdk/v3/proto/go"
 )
 
-var UseGrpcProvidersByDefault = false
+var UseGrpcPluginsByDefault = false
+
+type LoadPluginFunc func(opts interface{}) (interface{}, error)
+type LoadPluginWithHostFunc func(opts interface{}, host plugin.Host) (interface{}, error)
 
 type LoadProviderFunc func() (plugin.Provider, error)
 type LoadProviderWithHostFunc func(host plugin.Host) (plugin.Provider, error)
 
-type ProviderOption func(p *ProviderLoader)
+type LoadAnalyzerFunc func(opts *plugin.PolicyAnalyzerOptions) (plugin.Analyzer, error)
+type LoadAnalyzerWithHostFunc func(opts *plugin.PolicyAnalyzerOptions, host plugin.Host) (plugin.Analyzer, error)
 
-func WithoutGrpc(p *ProviderLoader) {
+type PluginOption func(p *PluginLoader)
+
+func WithoutGrpc(p *PluginLoader) {
 	p.useGRPC = false
 }
 
-func WithGrpc(p *ProviderLoader) {
+func WithGrpc(p *PluginLoader) {
 	p.useGRPC = true
 }
 
-type ProviderLoader struct {
-	pkg          tokens.Package
+type PluginLoader struct {
+	kind         workspace.PluginKind
+	name         string
 	version      semver.Version
-	load         LoadProviderFunc
-	loadWithHost LoadProviderWithHostFunc
+	load         LoadPluginFunc
+	loadWithHost LoadPluginWithHostFunc
 	useGRPC      bool
 }
 
+type ProviderOption = PluginOption
+type ProviderLoader = PluginLoader
+
 func NewProviderLoader(pkg tokens.Package, version semver.Version, load LoadProviderFunc,
 	opts ...ProviderOption) *ProviderLoader {
 
 	p := &ProviderLoader{
-		pkg:     pkg,
+		kind:    workspace.ResourcePlugin,
+		name:    string(pkg),
 		version: version,
-		load:    load,
-		useGRPC: UseGrpcProvidersByDefault,
+		load:    func(_ interface{}) (interface{}, error) { return load() },
+		useGRPC: UseGrpcPluginsByDefault,
 	}
 	for _, o := range opts {
 		o(p)
@@ -78,10 +89,41 @@ func NewProviderLoaderWithHost(pkg tokens.Package, version semver.Version,
 	load LoadProviderWithHostFunc, opts ...ProviderOption) *ProviderLoader {
 
 	p := &ProviderLoader{
-		pkg:          pkg,
+		kind:         workspace.ResourcePlugin,
+		name:         string(pkg),
 		version:      version,
-		loadWithHost: load,
-		useGRPC:      UseGrpcProvidersByDefault,
+		loadWithHost: func(_ interface{}, host plugin.Host) (interface{}, error) { return load(host) },
+		useGRPC:      UseGrpcPluginsByDefault,
+	}
+	for _, o := range opts {
+		o(p)
+	}
+	return p
+}
+
+func NewAnalyzerLoader(name string, load LoadAnalyzerFunc, opts ...PluginOption) *PluginLoader {
+	p := &PluginLoader{
+		kind: workspace.AnalyzerPlugin,
+		name: name,
+		load: func(optsI interface{}) (interface{}, error) {
+			opts, _ := optsI.(*plugin.PolicyAnalyzerOptions)
+			return load(opts)
+		},
+	}
+	for _, o := range opts {
+		o(p)
+	}
+	return p
+}
+
+func NewAnalyzerLoaderWithHost(name string, load LoadAnalyzerWithHostFunc, opts ...PluginOption) *PluginLoader {
+	p := &PluginLoader{
+		kind: workspace.AnalyzerPlugin,
+		name: name,
+		loadWithHost: func(optsI interface{}, host plugin.Host) (interface{}, error) {
+			opts, _ := optsI.(*plugin.PolicyAnalyzerOptions)
+			return load(opts, host)
+		},
 	}
 	for _, o := range opts {
 		o(p)
@@ -169,20 +211,22 @@ func (e *hostEngine) SetRootResource(_ context.Context,
 }
 
 type pluginHost struct {
-	providerLoaders []*ProviderLoader
+	pluginLoaders   []*ProviderLoader
 	languageRuntime plugin.LanguageRuntime
 	sink            diag.Sink
 	statusSink      diag.Sink
 
 	engine *hostEngine
 
-	providers map[plugin.Provider]io.Closer
+	providers []plugin.Provider
+	analyzers []plugin.Analyzer
+	plugins   map[interface{}]io.Closer
 	closed    bool
 	m         sync.Mutex
 }
 
 func NewPluginHost(sink, statusSink diag.Sink, languageRuntime plugin.LanguageRuntime,
-	providerLoaders ...*ProviderLoader) plugin.Host {
+	pluginLoaders ...*ProviderLoader) plugin.Host {
 
 	engine := &hostEngine{
 		sink:       sink,
@@ -201,12 +245,12 @@ func NewPluginHost(sink, statusSink diag.Sink, languageRuntime plugin.LanguageRu
 	engine.address = fmt.Sprintf("127.0.0.1:%v", port)
 
 	return &pluginHost{
-		providerLoaders: providerLoaders,
+		pluginLoaders:   pluginLoaders,
 		languageRuntime: languageRuntime,
 		sink:            sink,
 		statusSink:      statusSink,
 		engine:          engine,
-		providers:       map[plugin.Provider]io.Closer{},
+		plugins:         map[interface{}]io.Closer{},
 	}
 }
 
@@ -216,10 +260,12 @@ func (host *pluginHost) isClosed() bool {
 	return host.closed
 }
 
-func (host *pluginHost) Provider(pkg tokens.Package, version *semver.Version) (plugin.Provider, error) {
-	var best *ProviderLoader
-	for _, l := range host.providerLoaders {
-		if l.pkg != pkg {
+func (host *pluginHost) plugin(kind workspace.PluginKind, name string, version *semver.Version,
+	opts interface{}) (interface{}, error) {
+
+	var best *PluginLoader
+	for _, l := range host.pluginLoaders {
+		if l.kind != kind || l.name != name {
 			continue
 		}
 
@@ -238,19 +284,19 @@ func (host *pluginHost) Provider(pkg tokens.Package, version *semver.Version) (p
 
 	load := best.load
 	if load == nil {
-		load = func() (plugin.Provider, error) {
-			return best.loadWithHost(host)
+		load = func(opts interface{}) (interface{}, error) {
+			return best.loadWithHost(opts, host)
 		}
 	}
 
-	prov, err := load()
+	plug, err := load(opts)
 	if err != nil {
 		return nil, err
 	}
 
 	closer := nopCloser
 	if best.useGRPC {
-		prov, closer, err = wrapProviderWithGrpc(prov)
+		plug, closer, err = wrapProviderWithGrpc(plug.(plugin.Provider))
 		if err != nil {
 			return nil, err
 		}
@@ -259,8 +305,23 @@ func (host *pluginHost) Provider(pkg tokens.Package, version *semver.Version) (p
 	host.m.Lock()
 	defer host.m.Unlock()
 
-	host.providers[prov] = closer
-	return prov, nil
+	switch kind {
+	case workspace.AnalyzerPlugin:
+		host.analyzers = append(host.analyzers, plug.(plugin.Analyzer))
+	case workspace.ResourcePlugin:
+		host.providers = append(host.providers, plug.(plugin.Provider))
+	}
+
+	host.plugins[plug] = closer
+	return plug, nil
+}
+
+func (host *pluginHost) Provider(pkg tokens.Package, version *semver.Version) (plugin.Provider, error) {
+	plug, err := host.plugin(workspace.ResourcePlugin, string(pkg), version, nil)
+	if err != nil || plug == nil {
+		return nil, err
+	}
+	return plug.(plugin.Provider), nil
 }
 
 func (host *pluginHost) LanguageRuntime(runtime string) (plugin.LanguageRuntime, error) {
@@ -272,7 +333,7 @@ func (host *pluginHost) SignalCancellation() error {
 	defer host.m.Unlock()
 
 	var err error
-	for prov := range host.providers {
+	for _, prov := range host.providers {
 		if pErr := prov.SignalCancellation(); pErr != nil {
 			err = pErr
 		}
@@ -284,7 +345,7 @@ func (host *pluginHost) Close() error {
 	defer host.m.Unlock()
 
 	var err error
-	for _, closer := range host.providers {
+	for _, closer := range host.plugins {
 		if pErr := closer.Close(); pErr != nil {
 			err = pErr
 		}
@@ -308,13 +369,13 @@ func (host *pluginHost) LogStatus(sev diag.Severity, urn resource.URN, msg strin
 	}
 }
 func (host *pluginHost) Analyzer(nm tokens.QName) (plugin.Analyzer, error) {
-	return nil, errors.New("unsupported")
+	return host.PolicyAnalyzer(nm, "", nil)
 }
 func (host *pluginHost) CloseProvider(provider plugin.Provider) error {
 	host.m.Lock()
 	defer host.m.Unlock()
 
-	delete(host.providers, provider)
+	delete(host.plugins, provider)
 	return nil
 }
 func (host *pluginHost) ListPlugins() []workspace.PluginInfo {
@@ -330,9 +391,17 @@ func (host *pluginHost) GetRequiredPlugins(info plugin.ProgInfo,
 
 func (host *pluginHost) PolicyAnalyzer(name tokens.QName, path string,
 	opts *plugin.PolicyAnalyzerOptions) (plugin.Analyzer, error) {
-	return nil, errors.New("unsupported")
+
+	plug, err := host.plugin(workspace.AnalyzerPlugin, string(name), nil, opts)
+	if err != nil || plug == nil {
+		return nil, err
+	}
+	return plug.(plugin.Analyzer), nil
 }
 
 func (host *pluginHost) ListAnalyzers() []plugin.Analyzer {
-	return nil
+	host.m.Lock()
+	defer host.m.Unlock()
+
+	return host.analyzers
 }
