@@ -773,7 +773,7 @@ func (s *RefreshStep) Apply(preview bool) (resource.Status, StepCompleteFunc, er
 		s.new = resource.NewState(s.old.Type, s.old.URN, s.old.Custom, s.old.Delete, resourceID, inputs, outputs,
 			s.old.Parent, s.old.Protect, s.old.External, s.old.Dependencies, initErrors, s.old.Provider,
 			s.old.PropertyDependencies, s.old.PendingReplacement, s.old.AdditionalSecretOutputs, s.old.Aliases,
-			&s.old.CustomTimeouts, s.old.ImportID)
+			&s.old.CustomTimeouts, s.old.ImportID, s.old.SequenceNumber)
 	} else {
 		s.new = nil
 	}
@@ -915,7 +915,7 @@ func (s *ImportStep) Apply(preview bool) (resource.Status, StepCompleteFunc, err
 	// differences between the old and new states are between the inputs and outputs.
 	s.old = resource.NewState(s.new.Type, s.new.URN, s.new.Custom, false, s.new.ID, read.Inputs, read.Outputs,
 		s.new.Parent, s.new.Protect, false, s.new.Dependencies, s.new.InitErrors, s.new.Provider,
-		s.new.PropertyDependencies, false, nil, nil, &s.new.CustomTimeouts, s.new.ImportID)
+		s.new.PropertyDependencies, false, nil, nil, &s.new.CustomTimeouts, s.new.ImportID, s.new.SequenceNumber)
 
 	// If this step came from an import deployment, we need to fetch any required inputs from the state.
 	if s.planned {
@@ -946,7 +946,7 @@ func (s *ImportStep) Apply(preview bool) (resource.Status, StepCompleteFunc, err
 	}
 
 	// Check the inputs using the provider inputs for defaults.
-	inputs, failures, err := prov.Check(s.new.URN, s.old.Inputs, s.new.Inputs, preview)
+	inputs, failures, err := prov.Check(s.new.URN, s.old.Inputs, s.new.Inputs, preview, s.new.SequenceNumber)
 	if err != nil {
 		return rst, nil, err
 	}
@@ -1175,6 +1175,11 @@ func getProvider(s Step) (plugin.Provider, error) {
 	if err != nil {
 		return nil, fmt.Errorf("bad provider reference '%v' for resource %v: %v", s.Provider(), s.URN(), err)
 	}
+	if providers.IsDenyDefaultsProvider(ref) {
+		pkg := providers.GetDeniedDefaultProviderPkg(ref)
+		msg := diag.GetDefaultProviderDenied(s.URN()).Message
+		return nil, fmt.Errorf(msg, pkg, s.URN())
+	}
 	provider, ok := s.Deployment().GetProvider(ref)
 	if !ok {
 		return nil, fmt.Errorf("unknown provider '%v' for resource %v", s.Provider(), s.URN())
