@@ -25,6 +25,7 @@ import (
 	"github.com/stretchr/testify/require"
 	"pgregory.net/rapid"
 
+	"github.com/pulumi/pulumi/pkg/v3/secrets/b64"
 	"github.com/pulumi/pulumi/sdk/v3/go/common/apitype"
 	"github.com/pulumi/pulumi/sdk/v3/go/common/resource"
 	"github.com/pulumi/pulumi/sdk/v3/go/common/resource/config"
@@ -94,6 +95,7 @@ func TestDeploymentSerialization(t *testing.T) {
 		nil,
 		nil,
 		"",
+		0,
 	)
 
 	dep, err := SerializeResource(res, config.NopEncrypter, false /* showSecrets */)
@@ -403,6 +405,22 @@ func TestCustomSerialization(t *testing.T) {
 	})
 }
 
+func TestDeserializeDeploymentSecretCache(t *testing.T) {
+	urn := "urn:pulumi:prod::acme::acme:erp:Backend$aws:ebs/volume:Volume::PlatformBackendDb"
+	_, err := DeserializeDeploymentV3(apitype.DeploymentV3{
+		SecretsProviders: &apitype.SecretsProvidersV1{Type: b64.Type},
+		Resources: []apitype.ResourceV3{
+			{
+				URN:    resource.URN(urn),
+				Type:   "aws:ebs/volume:Volume",
+				Custom: true,
+				ID:     "vol-044ba5ad2bd959bc1",
+			},
+		},
+	}, DefaultSecretsProvider)
+	assert.NoError(t, err)
+}
+
 func TestDeserializeInvalidResourceErrors(t *testing.T) {
 	deployment, err := DeserializeDeploymentV3(apitype.DeploymentV3{
 		Resources: []apitype.ResourceV3{
