@@ -526,7 +526,8 @@ func (p *provider) Configure(inputs resource.PropertyMap) error {
 
 // Check validates that the given property bag is valid for a resource of the given type.
 func (p *provider) Check(urn resource.URN,
-	olds, news resource.PropertyMap, allowUnknowns bool) (resource.PropertyMap, []CheckFailure, error) {
+	olds, news resource.PropertyMap,
+	allowUnknowns bool, sequenceNumber int) (resource.PropertyMap, []CheckFailure, error) {
 	label := fmt.Sprintf("%s.Check(%s)", p.label(), urn)
 	logging.V(7).Infof("%s executing (#olds=%d,#news=%d", label, len(olds), len(news))
 
@@ -562,9 +563,10 @@ func (p *provider) Check(urn resource.URN,
 	}
 
 	resp, err := client.Check(p.requestContext(), &pulumirpc.CheckRequest{
-		Urn:  string(urn),
-		Olds: molds,
-		News: mnews,
+		Urn:            string(urn),
+		Olds:           molds,
+		News:           mnews,
+		SequenceNumber: int32(sequenceNumber),
 	})
 	if err != nil {
 		rpcError := rpcerror.Convert(err)
