@@ -158,7 +158,7 @@ export interface Provider {
      * @param olds The old input properties to use for validation.
      * @param news The new input properties to use for validation.
      */
-    check?: (urn: resource.URN, olds: any, news: any) => Promise<CheckResult>;
+    check?: (urn: resource.URN, olds: any, news: any, sequenceNumber: number) => Promise<CheckResult>;
 
     /**
      * Diff checks what impacts a hypothetical update will have on the resource's properties.
@@ -209,7 +209,7 @@ export interface Provider {
      * @param options the options for the resource.
      */
     construct?: (name: string, type: string, inputs: Inputs, options: resource.ComponentResourceOptions)
-        => Promise<ConstructResult>;
+    => Promise<ConstructResult>;
 
     /**
      * Call calls the indicated method.
