@@ -198,7 +198,7 @@ def get_root_resource() -> Optional['Resource']:
     """
     Returns the implicit root stack resource for all resources created in this program.
     """
-    global ROOT
+    global ROOT  # pylint: disable=global-variable-not-assigned
     return ROOT
 
 
@@ -223,7 +223,7 @@ async def monitor_supports_feature(feature: str) -> bool:
                 resp = monitor.SupportsFeature(req)
                 return resp.hasSupport
             except grpc.RpcError as exn:
-                if exn.code() != grpc.StatusCode.UNIMPLEMENTED: # pylint: disable=no-member
+                if exn.code() != grpc.StatusCode.UNIMPLEMENTED:  # pylint: disable=no-member
                     handle_grpc_error(exn)
                 return False
 
@@ -262,6 +262,10 @@ async def monitor_supports_resource_references() -> bool:
     return await monitor_supports_feature("resourceReferences")
 
 
+async def monitor_supports_output_values() -> bool:
+    return await monitor_supports_feature("outputValues")
+
+
 def reset_options(project: Optional[str] = None,
                   stack: Optional[str] = None,
                   parallel: Optional[int] = None,
